% Matlab Mistuning Code: Version 1.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
% *********************************************************
%
% File: forcing.m
% Purpose: Given spatial forcing mode l computes mistuned
%   forced response: X(z,w) = Inv[iwI-M(z)] Bl(z).
% Last Update: 04/27/98 by B.Shapiro
%
% *********************************************************

% Read in spatial forcing mode.
  fprintf('Please enter spatial forcing mode. It should vary between 0 and %g. \n',r-1);
  fprintf('(Note: -p nodal diameter forcing is given by l = %g-p.) \n',r);
  fprintf('\n');
  clear l;
    l = input('Spatial forcing mode: l = ');
    fprintf('\n');
    % Check input ok.
      while mod(l,1) != 0 | l < 0 | l > r-1
        fprintf('ERROR! Spatial forcing mode must be an integer between 0 and %g: \n',r-1);
        l = input('       l = ');
        fprintf('\n');
      end
    % End check.
  if l == 0
    l = r; % Convert to my indexing for 0ND.
  end
% Got spatial forcing mode: l between 1 and r.



% ---------------------------------------------------------------------
%   NOTE: Please see paper:                                           |
%    "Solving for Mistuned Forced Response by Symmetry", B. Shapiro   |
%          (submitted Journal of Propulsion and Power 03/98)          |
%   for a description of quantities g, gbar and Psi.                  |
% ---------------------------------------------------------------------


  % Find g0(j) = g_j(0).    % j between 1 and rd refers to jth mode.
    g0 = zeros(r*d,1);      % Only the forced mode l is active.
    g0((l-1)*d+1: l*d) = r*UTun( (l-1)*d+1:l*d , 1:d )*betaTun;
  % Got g0.


  % Find gbar(j)          % j between 1 and rd refers to jth mode.
    gbar = zeros(r*d,1);  % initialize.
    for q = 0:r-1
      topow = kron(1:r, ones(1,d) ) - l*ones(1,d*r); % [jth(1)-l, ..., jth(rd)-l].
      prow = p(q,r).^topow;                          % vector of p( (jth(j)-l)q ).
      DiagP = diag(prow);                            % Create diagonal matrix.
      deltaq = DiagP * ( Du(:,:,1+q)*betaTun + UTun(:,1:d)*Dbeta(:,1,1+q) );
      gbar = gbar + deltaq;         % contribution to gbar from qth sum component.
    end
  % Got gbar.


  % Find Psi0, DPsi: Psi(z) = Psi0 + DPsi.z + ...
    Psi0 = zeros(d,r*d);    % jth d-sized column for jth mode.
    DPsi = zeros(d,r*d,r);  % (:,j,i) is dPsi_j(z)/dzi.
    % Psi0
      for j = (l-1)*d+1: l*d
        Psi0(:,j) = g0(j)*VTun(1:d,j);  % got Psi(0) for jth mode.
      end
    % DPsi
      for j = 1:r*d
        for i = 1:r
          DPsi(:,j,i) = g0(j)*Dv(:,j,i)+gbar(j)*p((l-jth(j,d))*(i-1),r)*VTun(1:d,j);
          % got dPsi/dzi for jth mode.
        end 
      end
  % Got Psi0, PsiLin




% DESCRIPTION OF DATA STRUCTURES:
%   1) g0: Tuned contribution from left eigenvectors and
%          forcing vector beta: g0 = g(z=0).
%      Size: r d column vector.
%      Syntax: g0(j) gives contribution of jth mode/eigenvalue.
% 
%   2) gbar: Linear coefficients of g(z) (only need one by symmetry for
%          each mode/eigenvalue j). Here gbar = dg/dz1.
%      Size: r d column vector.
%      Syntax: gbar(j) gives contribution of jth mode/eigenvalue.
%
%   3) Psi0: Tuned contribution from left, right eigenvectors
%          and forcing vector beta: Psi0 = Psi(z=0) = g(z) v(z).
%      Size: d by rd matrix.
%      Syntax: Psi0(i,j) is ith element of contribution from jth mode.
%
%   4) DPsi: Linear coefficients of Psi(z), here DPsi(:,:,q) = dPsi/dzq.
%      Size: d by rd by r array.
%      Syntax: DPsi(i,j,q) is ith element of contribution from jth mode
%          of dPsi/dzq.





















