% Matlab Mistuning Code: Version 1.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
% *********************************************************
%
% File: show_fast.m
% Purpose: Given a mistuning vector z, this file plots
%   (using an exact solution of the equations at z) the
%     1) mistuning z 
%     2) tuned vs mistuned eigenvalues
%     3) tuned vs mistuned bode plots
%     4) tuned vs mistuned response at `worst' frequency.
%   This file requires variables from Mistuning.m, run
%   that first. (I may change this to saved file later.)
%   It is faster then show.m but only holds for PW0319 model.
% Last Update: 04/28/98 by B.Shapiro
%
% *********************************************************

% Run forcing.m first so spatial forcing l is well defined.

% Pick mistuning y:

  % Modal mistuning:
    md = 20;
    sz = 0.01;
    % y = sz * cos( (2*pi*md/r) * [1:1:r] );
      y = sz * randn(1,r);



% Initialize plots:
  figure(6);  % after first 5 ? of Mistuning.
  clf;


% Top left, mistuning z:
  subplot(2,2,1);
    plot(y,'bo-')
    title('Chosen mistuning pattern');
    xlabel('slot number');
    ylabel('mistuning');
      ymax = max(y);
      ymin = min(y);
      delta = max(ymax - ymin,1e-3);  
      ymax = ymax + delta/20;
      ymin = ymin - delta/20;
    axis([0 r+1 ymin ymax]);
% End top left.



  % Compute mistuned eigenvalues:
    % Construct M(y): (same as in Opt -> cost.m)
    zeta = 0.00032;
    for j = 0:r-1
      Mofy(j*d+1:(j+1)*d,:) = rot( MrowTun, r-j, d, r); % Tuned part.
        % Now modify by mistuning: elements (2,1) and (2,2).
          Mofy((j+1)*d,j*d+1) = Mofy((j+1)*d,j*d+1) ...
                              -2*y(j+1) -y(j+1)^2;
          Mofy((j+1)*d,j*d+2) = Mofy((j+1)*d,j*d+2) ...
                              -2*zeta*y(j+1);
    end
    % Got M(y).

    EigenMist = eig( Mofy );
  % Got mistuned eigenvalues.


% Top right, tuned vs mistuned eigenvalues:
  subplot(2,2,2);
    xmin = 1.25 * min( real( EigenTun ) );
    xmax = max( 0, max( real( EigenTun ) ) );
    ymax = max( imag( EigenTun ) );
    ymin = min( abs( imag( EigenTun ) ) );
    delta = ymax - ymin;
    ymax = ymax + delta/4;
    ymin = ymin - delta/4;
    plot( real(EigenTun), imag(EigenTun), 'r.', ...   % tuned eigenvalues
          real(EigenMist), imag(EigenMist), 'b+', ... % mistuned eigenvalues
            real(EigenTun( (l-1)*d+1:l*d )), ...      % the (tuned) forced mode.
            imag(EigenTun( (l-1)*d+1:l*d )), 'go' )
    title('Tuned vs Mistuned Eigenvalues');
    axis([xmin xmax ymin ymax]);
    xlabel('real');
    ylabel('imag');
% End top right.


 % I need to create the B(y) forcing vector:
   % For special case of PW model, B(y) = B(0) is easy.
     prow = p(l,r).^(0:1:r-1);    % [1,pl,pl^2,...,pl^(r-1)]
     B0 = kron(prow.',betaTun);    % B(0)
     Bofy = B0;                    % Not true in general.
 % Got B(y).





% Bottom left, tuned vs mistuned bode plots:
  subplot(2,2,3);
    % Generate data on X(0,w) and X(y,w)
      % number of points:
        pnts = 1e2;
        ymax = ymax + delta/2; % extend freq. axis.
        ymin = ymin - delta/2;
        inc = (ymax - ymin)/pnts;  % same limits as in eigenvalue plot.
        X0 = zeros(pnts,1);
        Xy = zeros(pnts,r);
        for cnt = 0:pnts
          w = ymin + cnt*inc;
            temp_X0 = XTun(l,w,EigenTun,Psi0,d);
            X0(cnt+1) = abs( temp_X0(1) );   % Change for efficient X(0,w)
            Xy(cnt+1,:) = abs( exact_resp(Mofy,Bofy,w,r,d).' );  % r entries.
        end
        w = ymin:inc:ymax;   % vector of freq
    % End generate data.

  % Plot bode:
    plot(w,X0,'r-',w,Xy,'b:');
    xmin = ymin; % set former y limits to x-axis, the freq.
    xmax = ymax; 
    X = [X0, Xy];
    ymin = min(min(X));
    ymax = max(max(X));
    dy = ymax - ymin;
    ymin = ymin - dy/10;
    ymax = ymax + dy/10;
    title('Tuned vs Mistuned Bodes');
    axis([xmin xmax ymin ymax]);
    xlabel('freq w');
    ylabel('amplitude |X|');
% End bottom left.    
 


  % Now I need to know the worst case freq:  
    [damping, numb] = max(real(EigenMist));
    worst_w = abs( imag(EigenMist(numb)) ); % It may chosen complex-conj.
  % Got worst case (approximately).


  % Define X(0,worst_w) and X(y,worst_w)
    clear X0 Xy
    X0_1 = XTun(l,worst_w,EigenTun,Psi0,d);  % Tuned resp. of blade 1.
    X0 = X0_1(1)*p(l,r).^(0:r-1);            % Generate all blades by sym.
    Xy = exact_resp(Mofy,Bofy,worst_w,r,d);
    mag = max(abs(Xy))/abs(X0(1));
    amp = ['Amp ' num2str(mag)];
  % Got X0 and Xy.



% Bottom right, tuned vs mistuned response at worst case freq:
  subplot(2,2,4);
    plot(real(X0), imag(X0), 'r.', real(Xy), imag(Xy),'bs'); 
    titlestr = ['Response at freq. w = ' num2str(worst_w)];
    title( titlestr );
    xlabel('real');
    ylabel('imag');
    xmax = 1.6*max(abs(Xy));
    axis([-xmax xmax -xmax xmax]);
    axis equal;
% End bottom right







% Comments for entire plot:
  addinfo(amp, date, 'red = tuned, blue = mistuned, green = forced.', ...
          'B. Shapiro, Caltech');

% Print plot to file:
  print -depsc Outputs/show_fast.eps


