% Matlab Mistuning Code: Version 2.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
%
% *********************************************************
%
% File: Mistuning.m
% Purpose: Main file, runs the mistuning code.
% Last Update: 06/11/98 by B.Shapiro
%
% *********************************************************

clear
tic

% Define GLOBAL variables:
    global r;        % number of blades
    global d;        % states per blade
    global EigenTun; % tuned eigenvalues
    global a;        % first order eigenval. pert.
    global b;        % sym. second order eigenval. pert.
    global c;        % asym. second order eigenval. pert.
    global PSI0;     % PSI(0) of coupling PSI(z)
    global DPSI;     % dPSI/dz of coupling PSI(z)
      % I have traded good programming practice (no global
      % variables) for computation speed. 04/29/98.
% End GLOBAL variables.


% Heading output:
  fprintf('\n')
  fprintf('\t M         M         A          SSSSSSS    TTTTTTTTTTTT \n')
  fprintf('\t MM       MM        AAA        SS     SS   T    TT    T \n')
  fprintf('\t MMM     MMM       AA AA       SS               TT \n')
  fprintf('\t MM M   M MM      AA   AA       SSSSSSS         TT \n')
  fprintf('\t MM  M M  MM     AAAAAAAAA            SS        TT \n')
  fprintf('\t MM   M   MM    AA       AA    SS     SS        TT \n')
  fprintf('\t MM       MM   AA         AA    SSSSSSS        TTTT \n')
  fprintf(' \n')
  fprintf('\t  MISTUNING     ANALYSIS   by   SYMMETRY    TECHNIQUES \n')
  fprintf(' \n')
  fprintf('\t \t Version 2.0, 11 June 1998. \n')
  fprintf('\n')
  fprintf('\t Created: Benjamin Shapiro \n')
  fprintf('\t \t  Control and Dynamical Systems \n')
  fprintf('\t \t  Caltech, Pasadena, CA 91125 \n')
  fprintf('\t \t  bshapiro@indra.caltech.edu \n')
  fprintf('\n')
  fprintf('\t Copyright (c) 1998 Benjamin Shapiro, Caltech. \n')
  fprintf('\n')
  fprintf('\n')




% ********************************************************
% USER INPUT: 
%
%  User defined flags:

   new = 1;  % 0 = load old model with old results.
             % 1 = evaluate new model and compute
             %     new results.

   hots = 0; % 0 = perturb eigenvectors/values to
             %     1rst/2nd order in mistuning z---
             %     leading order. (Will be inaccurate if
             %     [tuned eigenvalues almost indistinct
             %     compared to mistuning] equivalently 
             %     [if mistuning large compared to 
             %     aerodynamics].)
             % 1 = perturb eigenvectors/values to
             %     2nd/3rd order in mistuning z.
             %     Accuracy better but code takes more
             %     time to run. NOT ACTIVE RIGHT NOW,
             %     PLEASE WAIT FOR NEXT RELEASE: 04/22/98.
             %
             % NOTE: Only active if new = 1, otherwise will
             % use the hots value saved in old-model. If you
             % change hots you must rerun the model (new = 1).

   unix_machine = 1; % 0 = Not on a unix platform.
                     % 1 = On a unix machine:
                     %      Matlab will automatically delete old files
                     %      before writing new results. If you are on a
                     %      PC, Mac or some other platform, the files
                     %      "Outputs/StabData/Eigval:*" and " .....
                     %         .... "                  should be deleted
                     %      manually before each new run. Otherwise data 
                     %      will be written twice in each of the files 
                     %      "Eigval:i,j" and "****".

% END USER INPUT
% *********************************************************



% BEGIN GENERATE MODEL:
  if new 
    model                  % New model from file: model.m
  else
    load('Data/modelvar')  % Load old model from: Data/modelvar.mat
      fprintf('Loading model saved on %s. \n',savedon) 
      % Above load will reload the old value of hots.
  end

  % Output for model
  fprintf('\t Model has %g blades with %g states per blade. \n', r, d )
    if linmodel == 1 
       fprintf('\t It is linear in mistuning vector z. \n')
    else
       fprintf('\t It is nonlinear in mistuning vector z. \n')
    end
    if exist('savemodel') & savemodel 
       fprintf('\t Results saved to: Data/modelvar.mat \n')
    end
  fprintf('\n')

    % Time at finished model:
    etime = toc;
    fprintf('\t \t \t Elapsed time: %.3f minutes. \n', etime/60 )
    fprintf('\n')
% END GENERATE MODEL



% Print which perturbation method
  fprintf('Chosen perturbation method: \n')
    if hots
      fprintf('\t Eigenvectors/values will be perturbed to \n')
      fprintf('\t second/third order in mistuning z. \n')
      fprintf('\n')
    else
      fprintf('\t Eigenvectors/values will be perturbed to \n')
      fprintf('\t first/second order in mistuning z. \n')
      fprintf('\n')
    end



% BEGIN TUNED EIGENVALUE/VECTOR COMPUTATIONS:
% Compute tuned eigenval/vect:
  if new
    tuneigen       % Compute tuned eigenvalues/vectors.
  else
    load('Outputs/TunEig'); % Load old eigenvalues/vectors 
                            % from: Outputs/TunEig.mat
    fprintf('Loading tuned eigenvalues/vectors saved on %s. \n',savedon)
    fprintf('\t Tuned eigenvalue plot found in: Outputs/TunEigPlot.eps \n')
    fprintf('\t Tuned eigenvalues/vectors saved in: Outputs/TunEig.mat \n')
  end

    % Time when finished computing tuned eigenvalues/vectors:
    etime = toc;
    fprintf('\n')
    fprintf('\t \t \t Elapsed time: %.3f minutes. \n', etime/60 )
% END TUNED EIGENVALUE/VECTOR COMPUTATIONS.




% BEGIN EIGENVALUE/VECTOR PERTURBATIONS:
% Build mistuned eigenvalue/vector perturbations
  if new
    perturb      % Compute eigenvalue/vector perturbation coeff.
  else
    load('Outputs/PertEig'); % Load eigenval/vector pert. coeff.
    fprintf('\n')
    fprintf('Loading %s eigenvalue/vector perturbations. \n',savedon)
    fprintf('\t Got first/second order eigenvalue coefficients. \n')
    fprintf('\t Got first order left/right eigenvector coefficients. \n')
  end

    % Time when finished computing tuned eigenvalues/vectors:
    etime = toc;
    fprintf('\n')
    fprintf('\t \t \t Elapsed time: %.3f minutes. \n', etime/60 )
% END EIGENVALUE/VECTOR PERTURBATIONS.




% BEGIN STABILITY OUTPUT:
% Print stability output to file
  if new
    stability   % Write out stability output
  else
    % Do nothing: output already written.
      fprintf('\n');
      fprintf('Stability results: there are r*d = %g * %g = %g eigenvalues.\n',r,d,r*d);
      fprintf('\t Detailed perturbations saved in "Outputs/StabData" dir. \n');
      fprintf('\t See "Eigval:i,j" for jth eigenvalue of ith nodal-diameter. \n'); 
      fprintf('\t Destabilized eigenvalue plot found in: \n');
      fprintf('\t \t Outputs/DestEigPlot.eps \n');
  end
      
    % Time when finished writing stability data:
    etime = toc;
    fprintf('\n')
    fprintf('\t \t \t Elapsed time: %.3f minutes. \n', etime/60 )
% END STABILITY OUTPUT.





% BEGIN FORCED RESPONSE
  forcing;
    % Time when finished computing forced response:
    etime = toc;
    fprintf('\n')
    fprintf('\t \t \t Elapsed time: %.3f minutes. \n', etime/60 )
% END FORCED RESPONSE




% SHOW SOME PRETTY PICTURES
  % Picks a random or modal mistuning and plots output.
    show;
    % Time when finished show
    etime = toc;
    fprintf('\n')
    fprintf('\t \t \t Elapsed time: %.3f minutes. \n', etime/60 )
% END SHOW PRETTY PICTURES





% CLOSE
  fprintf('\n');  
  fprintf('\n');
  fprintf('\n');
  fprintf('Calculations complete! Look for additional packages/capabilities \n');
  fprintf('in future mistuning software versions. \n');
  fprintf('\n');
  fprintf('\t Questions/comments to: Benjamin Shapiro \n');
  fprintf('\t                        Control and Dynamical Systems \n');
  fprintf('\t                        Caltech (bshapiro@indra.caltech.edu). \n');
% END CLOSE


