% Matlab Mistuning Code: Version 2.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
%
% *********************************************************
%
% File: show_compare.m
% Purpose: Compare the exact response of show.m and the
%   approximate output generated in show_approx.m.
%   Given a mistuning vector z, this file plots the
%     1) mistuning z 
%     2) tuned vs mistuned (exact and approx) eigenvalues
%     3) tuned vs mistuned (exact and approx) bode plots
%        for blade one only (otherwise too many curves).
%     4) tuned vs mistuned (exact and approx) response 
%        at `worst' frequency.
%   This file loads variables from Mistuning.m.
%   (No need to run Mistuning if these saved files exist.)
% Last Update: 06/12/98 by B.Shapiro
%
% *********************************************************

% Load saved data:
  load('Data/modelvar');
  load('Outputs/TunEig');
  load('Outputs/forced');
  load('Outputs/PertEig');
% End load.




% Choose mistuning size:
  sz = 0.02;

  % Pick a random mistuning y:
    randn('state',sum(100*clock)); % new seed.
    y = sz * randn(1,r);
    rnd = 1;
 
  % or Pick a modal mistuning y:
  % md = 2;
  % y = sz * cos( (2*pi*md/r) * [1:1:r] );
  % rnd = 0;


% Print output:
  fprintf('\n');
  if rnd
    fprintf('Showing exact vs. approximate output for: \n');
    fprintf('\t random mistuning of size %g,\n',sz);
    fprintf('\t forcing in spatial mode %g. \n',l);
    fprintf('Plot saved to: Outputs/show_compare.eps. \n');
    fprintf('\n'); 
  else
    fprintf('Showing exact vs. approximate output for: \n');
    fprintf('\t modal mistuning of size %g, mode %g,\n',sz,md);
    fprintf('\t forcing in spatial mode %g. \n',l);
    fprintf('Plot saved to: Outputs/show_compare.eps. \n');
    fprintf('\n'); 
  end
% End print.



% Initialize plots:
  figure(3); 
  clf;



% Top left, mistuning z:
  subplot(2,2,1);
    plot(y,'bo-')
    title('Chosen mistuning pattern');
    xlabel('slot number');
    ylabel('mistuning');
      ymax = max(y);
      ymin = min(y);
      delta = max(ymax - ymin,1e-3);  
      ymax = ymax + delta/20;
      ymin = ymin - delta/20;
    axis([0 r+1 ymin ymax]);
% End top left.




% Top right, tuned vs mistuned eigenvalues:
  subplot(2,2,2);

  % Compute mistuned eigenvalues:
    % Construct M(y),B(y).
    clear Mofy;                    
      Mofy = zeros(r*d,r*d);  % Initialize   
      Bofy = zeros(r*d,1);    % Initialize  
    Mofy(1:d,:) = double( subs(Mrow,z,y) );
    Bofy(1:d,1) = double( subs(beta,z,y) );
    for j = 1:r-1
       Mofy(j*d+1:(j+1)*d,:) = ...
                double( rot( subs( Mrow, z, rot(y,j,1,r) )  ,r-j,d,r) );
       Bofy(j*d+1:(j+1)*d,1) = ...
                p(l*j,r)* double( subs(beta,z,rot(y,j,1,r)) );
    end
    % Got M(y),B(y).

    EigenMist = eig( Mofy );
  % Got mistuned eigenvalues.

  % Compute (approx) mistuned eigenvalues:
    EigenMistApp = ( EigenTun + EigenSec(y,a,b,c,r) ).';
  % Got (approx) mistuned eigenvalues.



  % Do the plot:
    xmin = 1.15 * min( real( EigenMist ) );
    xmax = max( 0, max( real( EigenMist ) ) );
    ymax = max( imag( EigenMist ) );
    ymin = min( abs( imag( EigenMist ) ) );
    delta = ymax - ymin;
    ymax = ymax + delta/6;
    ymin = ymin - delta/6;
    plot( real(EigenTun), imag(EigenTun), 'r.', ...         % tuned eigenvalues
          real(EigenMist), imag(EigenMist), 'b+', ...       % mistuned eigenvalues
          real(EigenMistApp), imag(EigenMistApp), 'mx', ... % approx mistuned eigenvalues
            real(EigenTun( (l-1)*d+1:l*d )), ...            % the (tuned) forced mode.
            imag(EigenTun( (l-1)*d+1:l*d )), 'go' )
    title('Tuned vs Mistuned Eigenvalues');
    axis([xmin xmax ymin ymax]);
    xlabel('real');
    ylabel('imag');
% End top right.



% Bottom left, tuned vs mistuned bode plots:
  subplot(2,2,3);
    % Generate data on X(0,w) and X(y,w)
      % number of points:
        pnts = 1e2;
        ymax = ymax + delta/2; % extend freq. axis.
        ymin = ymin - delta/2;
        inc = (ymax - ymin)/pnts;  % same limits as in eigenvalue plot.
        X0 = zeros(pnts,1);
        Xy = zeros(pnts,1);
        XyApp = zeros(pnts,1);
        for cnt = 0:pnts
          w = ymin + cnt*inc;
            temp_X0 = XTun(l,w,EigenTun,Psi0,d);
            X0(cnt+1) = abs( temp_X0(1) );   % X0(1) takes "position".
            Xy_temp = abs( exact_resp(Mofy,Bofy,w,r,d).' );  % r "position" entries.
            Xy(cnt+1) = Xy_temp(1);             % Only want first blade here.
            XyApp_temp = abs( Xsec(y,l,w).' );  % r "position" entries.
            XyApp(cnt+1) = XyApp_temp(1);       % Again, only first blade.
        end
        w = ymin:inc:ymax;   % vector of freq
    % End generate data.
 
  % Plot bode:
    plot(w,X0,'r-',w,Xy,'b-.',w,XyApp,'m-.');
    xmin = ymin; % set former y limits to x-axis, the freq.
    xmax = ymax; 
    X = [X0, Xy];
    ymin = min(min(X));
    ymax = max(max(X));
    dy = ymax - ymin;
    ymin = ymin - dy/10;
    ymax = ymax + dy/10;
    title('Tuned vs Mistuned Bodes');
    axis([xmin xmax ymin ymax]);
    xlabel('freq w');
    ylabel('amplitude |X|');
% End bottom left.    



% Bottom right, tuned vs mistuned response at worst case freq:
  subplot(2,2,4);

  % Now I need to know the (approximate) worst case freq:  
    [damping, numb] = max(real(EigenMist));
    worst_w = abs( imag(EigenMist(numb)) ); % It may chosen complex-conj.
  % Got worst case (approximately).


  % Define X(0,worst_w) and X(y,worst_w)
    clear X0 Xy
    X0_1 = XTun(l,worst_w,EigenTun,Psi0,d);  % Tuned response of blade 1.
    X0 = X0_1(1)*p(l,r).^(0:r-1);            % Generate all blades by sym.
    Xy = exact_resp(Mofy,Bofy,worst_w,r,d);  % again, r "position" entries.
    XyApp_temp = Xsec(y,l,worst_w).';        % all entries.
    XyApp = zeros(r,1);                      % initialize 
    for j = 1:r
      XyApp(j) = XyApp_temp( (j-1)*d +1 );   % Only r "position" entries.
    end
    mag = max(abs(Xy))/abs(X0(1));  % Again, X0(1) takes position.
    amp = ['Amp ' num2str(mag)];
  % Got X0 and Xy.


  % Do the plot:
    plot(real(X0), imag(X0), 'r.', real(Xy), imag(Xy),'bs', ...
         real(XyApp), imag(XyApp), 'm^'); 
    titlestr = ['Response at freq. w = ' num2str(worst_w)];
    title( titlestr );
    xlabel('real');
    ylabel('imag');
    xmax = 1.6*max(abs(Xy));
    axis([-xmax xmax -xmax xmax]);
    axis equal;
% End bottom right





% Comments for entire plot:
  addinfo(amp, date, ...
          'red = tuned, blue = exact mistuned, magenta = approx. mistuned, green = forced.', ...
          'MAST: B. Shapiro, Caltech');
 
% Print plot to file:
  print -depsc Outputs/show_compare.eps





% Aside: This piece of code automatically displays the first element 
%   for the state of each blade (the "position", not the "velocity").
%   For the exact mistuned response, exact_resp is wired this way also.
%   If I want to change to looking at velocity I need to change two
%   pieces of code (this one and exact_resp).    B. Shapiro, 05/01/98