% Matlab Mistuning Code: Version 2.0
%
% Created by: Benjamin Shapiro,
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
%
% *********************************************************
%
% File: stability.m
% Purpose: Output eigenvalue stability results.
% Last Update: 06/12/98 by B.Shapiro
%
% *********************************************************


% Heading output:
  fprintf('\n')
  fprintf('Stability results: there are r*d = %g * %g = %g eigenvalues.\n',r,d,r*d);
  % First delete old files: 
    if unix_machine
      fprintf('\t (Delete old files: Outputs/StabData/Eigenval:*.) \n');
      unix('rm -f Outputs/StabData/Eigval:*');
    end
  % Only active on Unix platforms.
  fprintf('\t Detailed perturbations written in "Outputs/StabData" dir. \n');
  fprintf('\t See "Eigval:i,j" for jth eigenvalue of ith nodal-diameter. \n'); 


  
    % Clear space for eigenvalues of Re[Sj]
    ReigS = zeros(r*d,K+1);    % jth row contains K+1 eigenvalues of Re[Sj].
                               % note: S has repeating eigenvalues. So, only K+1
                               %       distinct eigenvalues: K = r/2 or (r-1)/2.
    can_destab = zeros(1,r*d); % for jth eigval, counts no. destab. mist. modes.

    
% Loop over nodal diameter:
  for ND = 1:r
    % Loop within d
    for i = 1:d
      % CONSIDER (ND,i)th EIGENVALUE:
      % In regular indexing: (ND,i) -> j.
        j = (ND-1)*d+i;

        % Compute eigenvalues of Re[Sj]
          for s = 0:K
            % Important: shifted numbering by one.
            % Eigenvalue p0 of Re[Sj] is first entry (for average mist.)
            % p1 is second entry, etc.
             ReigS(j,s+1) = real( b(j) ) ...
                          + real( c(j,:) )*cos( 2*pi*s*(1:1:K)/r ).';
          end     
        % Got eigenvalues of Re[Sj].

        % Are there any destabilizing directions (excluding average) ?
          Destab =  ReigS(j,2:K+1) > zeros(1,K);
          can_destab(1,j) = sum( Destab );
        % Here "can_destab" lists number of mistuning modes that cause
        % (ND,i)th (equiv. jth) eigenvalue to move right.

        % Create file to write results to:
          clear fid;
          filename = ['Outputs/StabData/Eigval:' num2str(ND) ',' num2str(i) ];
          fid = fopen(filename,'at');   
         
        % Write results to file: Outputs/StabData/Eigval:ND,i
          fprintf(fid,'\n');
          fprintf(fid,'\t PERTURBING EIGENVALUE %g OF NODAL-DIAMETER %g. \n', i,ND );
          fprintf(fid,'\t    (or: eigenvalue %g in regular indexing) \n',j);
          fprintf(fid,'\n');
          fprintf(fid,'Tuned eigenvalue (at z=0) is: lambda(0) = ');
          fprintf(fid,'%g + %g i.\n', real(EigenTun(j)), imag(EigenTun(j)) );
          fprintf(fid,'\n');
          fprintf(fid,'Eigenvalue perturbations given by:\n');
          fprintf(fid,'\n');
          fprintf(fid,'\t lambda(z) = lambda(0) \n');
          fprintf(fid,'\t           + a  ( z1   +  z2   + ... +  zr   ) _  \n');
          fprintf(fid,'\t           + b  (z1^2  + z2^2  + ... + zr^2  )  | \n');
          fprintf(fid,'\t           + c1 (z1*z2 + z2*z3 + ... + zr*z1 )  | \n');
          fprintf(fid,'\t           + c2 (z1*z3 + z2*z4 + ... + zr*z2 )  > = z^T S z\n');
          fprintf(fid,'\t             :     :      :             :       | \n');
          fprintf(fid,'\t           + cK ( z1*z[1+K]    + ... + zr*zK ) _| \n');
          fprintf(fid,'\t           + O( ||z||^3 ).\n');
          fprintf(fid,'\n');
          fprintf(fid,'COEFF: ');
            fprintf(fid,'a = %f + %f i, ', real(a(j)), imag(a(j)) );   % a
            fprintf(fid,'b = %f + %f i, \n', real(b(j)), imag(b(j)) ); % b
            fprintf(fid,'       ');
            for q = 1:K
              fprintf(fid,'c%g = %f + %f i, ', q,real(c(j,q)), imag(c(j,q)) ); % cq
              if ~mod(q,2) 
                 fprintf(fid,'\n       '); % line-break   
              end
            end
          fprintf(fid,'\n \n');
          fprintf(fid,'STAB/DESTAB MISTUNING:\n');
          fprintf(fid,'   To leading order, matrix Re[S] determines stabilizing/ \n');
          fprintf(fid,'   destabilizing mistuning directions. Specifically, mode \n');
          fprintf(fid,'   j mistuning (zi = cos(2 pi i j/r) is stab/destab if pj \n');
          fprintf(fid,'   (the jth eigenvalue of Re[S]) is negative/positive. \n');
          fprintf(fid,'   Note: Re[S] only has K+1 = %g distinct eigenvalues. \n', K+1 );
          fprintf(fid,'\n');
          fprintf(fid,'Eig(Re[S]): ');
            for q = 0:K
              fprintf(fid,'p%g = %f, ', q, ReigS(j,q+1) ); % pq
              if ~mod(q+1,3) 
                 fprintf(fid,'\n            '); % line-break   
              end
            end
          fprintf(fid,'\n');
          fprintf(fid,'Not counting average mistuning (mode j=0, p0) there are %g\n', ... 
                       can_destab(j) );
          fprintf(fid,'destabilizing directions (out of a possible %g). \n \n', K );
          fprintf(fid,'                ----------*---------- \n');
          fprintf(fid,'\n');
          fprintf(fid,'See "A Symmetry Approach to Extension of Flutter \n');
          fprintf(fid,'     Boundaries via Mistuning", B. Shapiro for derivation.\n');
          fprintf(fid,'\n');
          fprintf(fid,'Copyright: B. Shapiro, CDS, Caltech. \nCreated: ');
          fprintf(fid,date);
          fprintf(fid, '\n \n');
          fclose(fid); 
        % End write results to file:
          
      % END (ND,i)th EIGENVALUE.
    end
    % end d loop
  end
% end ND loop




% Plot destab. modes per eigenvalue:
xmin = 1.1 * min( real( EigenTun ) );
xmax = max( 0, max( real( EigenTun ) ) );
ymax = max( imag( EigenTun ) );
ymin = min( abs( imag( EigenTun ) ) );
delta = ymax - ymin;
ymax = ymax + delta/10;
ymin = ymin - delta/10;
  figure(2);
  clf;
  plot( EigenTun, 'r*')
  title('Number destabilizing mistuning modes per eigenvalue.');
  axis([xmin xmax ymin ymax]);
  xlabel('real');
  ylabel('imag');
  mdstr = [ 'max ' num2str(K) ' modes'];
  addinfo(mdstr,date,'Complex conjugates not shown.','MAST: B. Shapiro, Caltech');
  % Number eigenvalues by nodal diameter
    for j = 1:r*d
        x = real( EigenTun(j) );
        y = imag( EigenTun(j) );
        y = y + delta/40;
        h = text(x,y,num2str(can_destab(1,j)));
        set(h,'horizontalalignment','center')
    end
clear xmin xmax ymin ymax delta x y h j k;
% End plot destab. modes per eigenvalue.
% Print plot to file:
  print -depsc Outputs/DestEigPlot.eps
  fprintf('\t Destabilized eigenvalue plot saved to: \n')
  fprintf('\t \t Outputs/DestEigPlot.eps \n')


% Save results:
  savedon = date;
  save('Outputs/stab','ReigS','can_destab','savedon');






% DESCRIPTION OF DATA STRUCTURES:
%   1) ReigS:  Eigenvalues of Re[S] for each eigenvalue lambda_j.
%              As in: lambda_j(z) = lambda_j(0) + aj Sum(zi) + z' Sj z.
%      Size:   K+1 (columns) by r d (rows).
%      Syntax: ReigS(j,k) lists kth eigenvalue of Re[Sj]. Each matrix Sj
%              has K+1 non-distinct eigenvalues: K = r/2 or (r-1)/2.
%   2) can_destab: Counts number of destabilizing directions (excluding
%              average direction z = (1,1,..,1)) for matrix Re[Sj]. 
%              There are K such directions: if can_destab(j) = 0 then
%              all mistuning is stabilizing (to leading order) whereas
%              if can_destab(j) = K all mistuning is destabilizing.
%      Size: r d row vector.
%      Syntax: can_destab(j) is number destab. directions of Re[Sj].