% Matlab Mistuning Code: Version 2.0
%  
% Created by: Benjamin Shapiro, 
%             Control and Dynamical Systems
%             California Institute of Technology
%             Pasadena, CA 91125.
%             bshapiro@indra.caltech.edu
%
%   Copyright (c) 1998, California Institute of Technology.
%   Permission is granted to copy, modify, and redistribute
%   this software provided all header messages are retained.
% 
% *********************************************************
% 
% File: tuneigen.m 
% Purpose: Computes tuned eigenvalues and eigenvectors. 
% Last Update: 06/12/98 by B.Shapiro
%
% *********************************************************

fprintf('Computing tuned eigenvalues/vectors ... \n')

  % Clear space for data structures:
  UTun = zeros(d*r,d*r);   % Tuned left eigenvectors.
  VTun = zeros(d*r,d*r);   % Tuned right eigenvectors.
  Q = zeros(d,d,r);        % Qj matrices.
  rev = zeros(d,d,r);      % right (block) eigenvectors
  lev = zeros(d,d,r);      % left (block) eigenvectors
  pblock = zeros(d,r*d);   % temp variable
  EigenTun = zeros(1,r*d); % List of eigenvalues.
  prow = zeros(1,r);       % temp variable.
  % Largest first to optimize memory.


% Construct Qj matrices, find tuned eigenvalues/vectors:
for j = 1:r
  % Find Qj:
  % Qj = M1 + pj M2 + pj^2 M3 + ... + pj^(r-1) Mr
    prow = p(j,r).^(0:1:r-1);
    pblock = kron(prow, eye(d) );
    Q(:,:,j) = MrowTun*pblock.';  % .' is non-conjugate transpose.
 


if d == 2
  % Compute eigenvalues/vectors by analytic formulas.
  % Faster and more accurate. 
    X = eig2( Q(:,:,j) );
    % The eigenvalues
    EigenTun((j-1)*d+1: j*d) = X(:,1);
    % Right eigenvectors (blocks)
    rev(:,:,j) = X(:,2:3);
    % Left eigenvectors (blocks)
    lev(:,:,j) = X(:,4:5);    
  % Ideally, this could be extended up to d = 5,
  % d=1 case trivial, d=5 is last case where closed
  % formulas available (roots of order 5 polynomial).
  % End analytic formulas.
else
  % Compute eigenvalues/vectors by Matlab numerics 
  % The eigenvalues:
    EigenTun((j-1)*d+1: j*d) = eig( Q(:,:,j) );
  % Right eigenvectors (blocks)
    [rights, LambdaR] = eig( Q(:,:,j) );
    rev(:,:,j) = rights;
  % Left eigenvectors (blocks)
    % I could compute these by evaluating [lefts, LambdaL] = eig( Q(:,:,j)' ); 
    % lev(:,:,j) = lefts; but would have to match order (time consuming).
    lev(:,:,j) = inv( rev(:,:,j) );
  % End Matlab numerics.
end


    % Check to see if ill-conditioned eigenvectors:
      illcond = 1e5;
      if norm( lev(:,:,j) ) > illcond 
        fprintf('\t Warning: mode %g eigenvectors illconditioned. \n',j)
        fprintf('\t \t (To be fixed in software update.) \n')
        % Note: each mode j contains d left/right eigenvect.
      end
    % End check ill-conditioned.


% Compuse full left, right eigenvectors:
  % Right eigenvectors (full)
    VTun(:,(j-1)*d+1:j*d) = (pblock.')*rev(:,:,j) / sqrt(r);
    % Normalized to ||VTun(:,l)|| = 1.

  % Left eigenvectors (full)
    UTun((j-1)*d+1:j*d,:) = lev(:,:,j)*conj( pblock ) / sqrt(r);
    % Normalized so VTun*UTun = I.      
end




fprintf('\t \t \t \t \t \t done! \n')

% Plot tuned eigenvalues:
xmin = 1.1 * min( real( EigenTun ) );
xmax = max( 0, max( real( EigenTun ) ) );
ymax = max( imag( EigenTun ) );
ymin = min( abs( imag( EigenTun ) ) );
delta = ymax - ymin;
ymax = ymax + delta/10;
ymin = ymin - delta/10;
  figure(1);
  clf;
  plot( EigenTun, 'r*')
  title('Tuned Eigenvalues: numbered by nodal diameter.');
  axis([xmin xmax ymin ymax]);
  xlabel('real');
  ylabel('imag');
  addinfo('',date,'Complex conjugates not shown.','MAST: B. Shapiro, Caltech');
  % Number eigenvalues by nodal diameter
    for j = 1:r
      for k = 1:d
        x = real( EigenTun( (j-1)*d + k) );
        y = imag( EigenTun( (j-1)*d + k) );
        y = y + delta/40;
        h = text(x,y,num2str(j));
        set(h,'horizontalalignment','center')
      end
    end
clear xmin xmax ymin ymax delta x y h j k;
% End plot tuned eigenvalues.
% Print plot to file:
  print -depsc Outputs/TunEigPlot.eps
  fprintf('\t Tuned eigenvalue plot saved to: Outputs/TunEigPlot.eps \n')
  fprintf('\t Tuned eigenvalues/vectors saved to: Outputs/TunEig.mat \n')

% Save tuned eigenvalues/vectors: 
  savedon = date;
  save('Outputs/TunEig','EigenTun','UTun','VTun','savedon')




% DESCRIPTION OF DATA STRUCTURES:
%   1) Q: Qj = M1 + pj M2 + pj^2 M3 + .. + pj^(r-1) Mr
%         at z = 0. Eigenvalues Qj's = eigenvalues of M(0).
%      Size: r of d by d matrices.
%      Syntax: Q(a,b,j) is a (row) b (column) element of Qj.
%
%   2) VTun: Matrix of right (column) eigenvectors of M(0).
%            Column (j-1)d+i gives ith r.e.v for jth nodal diam.
%            Normalized so each column Vj has ||Vj|| = 1.
%      Size: rd by rd matrix.
%      Syntax: VTun(i,j) denotes ith row, jth column.
%
%   3) UTun: Matrix of left (row) eigenvectors of M(0).
%            Row (j-1)d+i gives ith l.e.v for jth nodal diam.
%            Inverse of VTun: VTun*UTun = UTun*VTun = I.
%            (Unless some l.e.v's ill-conditioned => zeroed.)
%      Size: rd by rd matrix.
%      Syntax: UTun(i,j) denotes ith row, jth column.
%
%   3) EigenTun: List of tuned eigenvalues of M(0). Element
%            (j-1)d+i gives ith e.v. for jth nodal diam.
%      Size: rd vector.
%      Syntax: EigenTun(i) denotes ith tuned eigenvalue.
%






% REMARKS:
%   Eigenvectors may be ill-conditioned (almost linearly
%   dependent)---very rare! In that case can either
%     a) zero out errant left eigenvectors (first cut)
%        and accept associated error (small if zeroed
%        vectors corresponding to high damped modes).        
%     b) can try different decomposition, say Schur.
%   If relevent problem, will fix in later versions. 03/25/98.
%
%   For d = 2 (two states per blade) I use the analytic 
%   formulas of "eig2.m" to solve for eigenvalues and 
%   eigenvectors. This is to increase accuracy and speed.
%   In prinicipal, one could write functions "eig3.m", 
%   "eig4.m" and "eig5.m" because there exist closed form
%   expressions for the roots (eigenvalues) of polynomials
%   up to order five. For d > 5 one has to rely on numerical
%   schemes. However, the analytic formulas get progressively
%   more complex and less wortwhile. 04/05/98.
